#!/usr/bin/env python
from typing import List
MIDDLE_EAST_REGIONS: List[str] = [
    'IRAQ',
    'TURKEY',
    'SYRIA',
    'YEMEN',
    'SAUDI ARABIA',
    'LEBANON',
    'PALESTINE',
    'IRAN',
    'ISRAEL',
    'UNITED ARAB EMIRATES',
    'BAHRAIN',
    'JORDAN',
    'OMAN',
    'QATAR',
    'KUWAIT'
]